@@ -3,7 +3,7 @@
 #
 #    conversion to class "im"
 #
-#    $Revision: 1.59 $   $Date: 2021/06/30 08:23:24 $
+#    $Revision: 1.63 $   $Date: 2025/09/03 23:18:54 $
 #
 #    as.im()
 #
@@ -12,11 +12,16 @@ as.im <- function(X, ...) {
   UseMethod("as.im")
 }
 
+as.im.NAobject <- function(X, ...) { NAobject("im") }
+
 as.im.im <- function(X, W=NULL, ...,
                      eps=NULL, dimyx=NULL, xy=NULL,
+                     rule.eps=c("adjust.eps", "grow.frame", "shrink.frame"),
                      na.replace=NULL) {
   X <- repair.old.factor.image(X)
   nopar <- is.null(eps) && is.null(dimyx) && is.null(xy)
+  if(!nopar)
+    rule.eps <- match.arg(rule.eps)
   if(is.null(W)) {
     if(nopar) {
       X <- repair.image.xycoords(X)
@@ -24,12 +29,13 @@ as.im.im <- function(X, W=NULL, ...,
       return(X)
     }
     # pixel raster determined by dimyx etc
-    W <- as.mask(as.rectangle(X), eps=eps, dimyx=dimyx, xy=xy)
+    W <- as.mask(as.rectangle(X),
+                 eps=eps, dimyx=dimyx, xy=xy, rule.eps=rule.eps)
     # invoke as.im.owin
     Y <- as.im(W)
   } else if(is.mask(W) || is.im(W) || !nopar) {
     #' raster information is present in { W, eps, dimyx, xy }
-    Y <- as.im(W, eps=eps, dimyx=dimyx, xy=xy)
+    Y <- as.im(W, eps=eps, dimyx=dimyx, xy=xy, rule.eps=rule.eps)
   } else {
     #' use existing raster information in X
     return(X[W, drop=FALSE, tight=TRUE])
@@ -41,16 +47,22 @@ as.im.im <- function(X, W=NULL, ...,
 
 as.im.owin <- function(X, W=NULL, ...,
                        eps=NULL, dimyx=NULL, xy=NULL,
+                       rule.eps=c("adjust.eps", "grow.frame", "shrink.frame"),
                        na.replace=NULL, value=1) {
   if(!(is.null(eps) && is.null(dimyx) && is.null(xy))) {
-    # raster dimensions determined by dimyx etc
-    # convert X to a mask 
-    M <- as.mask(X, eps=eps, dimyx=dimyx, xy=xy)
-    # convert mask to image
+    ## raster dimensions determined by dimyx etc
+    ## convert X to a mask
+    rule.eps <- match.arg(rule.eps)
+    M <- as.mask(X, eps=eps, dimyx=dimyx, xy=xy, rule.eps=rule.eps)
+    ## convert mask to image
     d <- M$dim
     v <- matrix(value, d[1L], d[2L])
+    ## factor case
+    levels(v) <- levels(value)
+    ## outside window
     m <- M$m
     v[!m] <- if(is.null(na.replace)) NA else na.replace
+    ## create
     out <- im(v, M$xcol, M$yrow,
               xrange=M$xrange, yrange=M$yrange,
               unitname=unitname(X))
@@ -69,6 +81,7 @@ as.im.owin <- function(X, W=NULL, ...,
     } else {
       # map {0, 1} to {na.replace, value}
       v <- matrix(ifelseAB(Z$v == 0, na.replace, value), d[1L], d[2L])
+      levels(v) <- levels(value)
       out <- im(v, W$xcol, W$yrow, unitname=unitname(X))
     }
     return(out)
@@ -80,6 +93,7 @@ as.im.owin <- function(X, W=NULL, ...,
     v <- matrix(value, d[1L], d[2L])
     m <- X$m
     v[!m] <- if(is.null(na.replace)) NA else na.replace
+    levels(v) <- levels(value)
     out <- im(v, xcol=X$xcol, yrow=X$yrow,
               xrange=X$xrange, yrange=X$yrange, unitname=unitname(X))
     return(out)
@@ -91,6 +105,7 @@ as.im.owin <- function(X, W=NULL, ...,
   # convert mask to image
   d <- M$dim
   v <- matrix(value, d[1L], d[2L])
+  levels(v) <- levels(value)
   m <- M$m
   v[!m] <- if(is.null(na.replace)) NA else na.replace
   out <- im(v, M$xcol, M$yrow, unitname=unitname(X))
@@ -103,6 +118,7 @@ as.im.funxy <- function(X, W=Window(X), ...) {
 
 as.im.function <- function(X, W=NULL, ...,
                            eps=NULL, dimyx=NULL, xy=NULL,
+                           rule.eps=c("adjust.eps", "grow.frame", "shrink.frame"),
                            na.replace=NULL,
                            stringsAsFactors=NULL,
                            strict=FALSE, drop=TRUE) {
@@ -111,7 +127,8 @@ as.im.function <- function(X, W=NULL, ...,
     stop("A window W is required")
   stringsAsFactors <- resolve.stringsAsFactors(stringsAsFactors)
   W <- as.owin(W)
-  W <- as.mask(W, eps=eps, dimyx=dimyx, xy=xy)
+  rule.eps <- match.arg(rule.eps)
+  W <- as.mask(W, eps=eps, dimyx=dimyx, xy=xy, rule.eps=rule.eps)
   m <- W$m
   funnywindow <- !all(m)
 
@@ -197,13 +214,15 @@ as.im.matrix <- function(X, W=NULL, ...) {
 
 as.im.default <- function(X, W=NULL, ...,
                           eps=NULL, dimyx=NULL, xy=NULL,
+                          rule.eps=c("adjust.eps", "grow.frame", "shrink.frame"),
                           na.replace=NULL) {
-
+  rule.eps <- match.arg(rule.eps)
   if((is.vector(X) || is.factor(X)) && length(X) == 1) {
     # numerical value: interpret as constant function
     xvalue <- X
     X <- function(xx, yy, ...) { rep.int(xvalue, length(xx)) }
-    return(as.im(X, W, ..., eps=eps, dimyx=dimyx, xy=xy, na.replace=na.replace))
+    return(as.im(X, W, ..., eps=eps, dimyx=dimyx, xy=xy,
+                 rule.eps=rule.eps, na.replace=na.replace))
   }
   
   if(is.list(X) && checkfields(X, c("x","y","z"))) {
@@ -229,9 +248,9 @@ as.im.default <- function(X, W=NULL, ...,
     out <- im(t(z), x, y)
     # now apply W and dimyx if present
     if(is.null(W) && !(is.null(eps) && is.null(dimyx) && is.null(xy)))
-      out <- as.im(out, eps=eps, dimyx=dimyx, xy=xy)
+      out <- as.im(out, eps=eps, dimyx=dimyx, xy=xy, rule.eps=rule.eps)
     else if(!is.null(W))
-      out <- as.im(out, W=W, eps=eps, dimyx=dimyx, xy=xy)
+      out <- as.im(out, W=W, eps=eps, dimyx=dimyx, xy=xy, rule.eps=rule.eps)
     return(na.handle.im(out, na.replace))
   }
   stop("Can't convert X to a pixel image")
@@ -306,8 +325,11 @@ as.im.data.frame <- function(X, ..., step, fatal=TRUE, drop=TRUE) {
 
 do.as.im <- function(x, action, ...,
                      W = NULL, eps = NULL, dimyx = NULL, xy = NULL, 
+                     rule.eps=c("adjust.eps", "grow.frame", "shrink.frame"),
                      na.replace = NULL) {
-  Z <- as.im(x, W=W, eps=eps, dimyx=dimyx, xy=xy, na.replace=na.replace)
+  rule.eps <- match.arg(rule.eps)
+  Z <- as.im(x, W=W, eps=eps, dimyx=dimyx, xy=xy,
+             rule.eps=rule.eps, na.replace=na.replace)
   Y <- do.call(action, list(Z, ...))
   return(Y)
 }
@@ -361,9 +383,13 @@ wrangle2image <- function(values, template) {
   template <- as.im(template)
   tdim <- dim(template)
   nt <- prod(tdim)
-  vdim <- dim(values) # may be NULL
+  if(!is.null(vlev <- levels(values))) {
+    ## flatten and convert to integer
+    if(!is.factor(values)) values <- factor(values, levels=vlev)
+    values <- as.integer(values)
+  } 
   nv <- length(values)
-  vlev <- levels(values)
+  vdim <- dim(values) # may be NULL
   vnames <- NULL
   if(is.null(vdim)) {
     ## vector or factor of values 
@@ -393,17 +419,27 @@ wrangle2image <- function(values, template) {
   }
   ## values is now a matrix or array
   if(is.matrix(values)) {
-    result <- template
-    result[drop=FALSE] <- values
+    if(!is.null(vlev)) values <- factor(values, labels=vlev)
+    result <- im(values,
+                 xcol=template$xcol,
+                 yrow=template$yrow,
+                 xrange=template$xrange,
+                 yrange=template$yrange,
+                 unitname=unitname(template))
     result <- result[W, drop=FALSE]
   } else {
     m <- dim(values)[3L]
     result <- vector(mode="list", length=m)
     for(i in seq_len(m)) {
-      Z <- template
-      Z[drop=FALSE] <- values[,,i]
-      Z <- Z[W, drop=FALSE]
-      result[[i]] <- Z
+      vi <- values[,,i]
+      if(!is.null(vlev)) vi <- factor(vi, labels=vlev)
+      Z <- im(vi,
+              xcol=template$xcol,
+              yrow=template$yrow,
+              xrange=template$xrange,
+              yrange=template$yrange,
+              unitname=unitname(template))
+      result[[i]] <- Z[W, drop=FALSE]
     }
     names(result) <- vnames
     result <- as.solist(result)
